# -*- coding: utf-8 -*-
from irc3.plugins.command import command
from datetime import datetime
import operator
import irc3
import timeago
import dateutil.parser
###########################################################################################
###########################################################################################
@irc3.plugin
class Plugin:
    #######################################################################################
    #######################################################################################
    def __init__(self, bot):
        self.bot = bot
        return
    #######################################################################################
    #######################################################################################
    @irc3.event(irc3.rfc.NEW_NICK)
    def on_nick_change_for_seen(self, nick, new_nick):
        # record of nickchanges, these are kept
        # "5": {
        # "msg": "_d changed nick to d",
        # "time": "2021-10-03T00:24:03.040916",
        # "type": "newnick"
        # },
        new_key = "last_msg_for_{}".format(new_nick).lower()
        msg_list = self.bot.db.getlist(new_key)
        if not msg_list:
            msg_list = []
        seen_msg = { "type": "newnick",
                "time": datetime.now().isoformat(),
                "msg": "{} changed nick to {}".format(nick.nick, new_nick)}
        msg_list.append(seen_msg)
        self.bot.db.setlist(new_key, msg_list)
        return
    #######################################################################################
    #######################################################################################
    @irc3.event(irc3.rfc.PRIVMSG)
    def on_privmsg_for_seen(self, mask=None, target=None, data=None, **kw):
        if data.startswith("?"): return # no ?
        if mask.lnick == self.bot.get_nick().lower(): return # Ignore ourseves
        key = "last_msg_for_{}".format(mask.nick).lower()
        messages = self.bot.db.getlist(key)
        # record of did a user speak this day and if so how many words spoken
        # if there is a record that day, they spoke. if there is no record for that
        # day then they didn't speak. this data is used as historicals to discern
        # true lurkers from idlers which are two very different things entirely.
        # <<< this record specifically will be used in  plugins/dsa_plugin.py >>>
        # record created:
        # "41": {
        #     "msg": 5,
        #     "time": "2022-07-27",
        #     "type": "activity"
        # },
        words_count=0
        if not messages:
            messages = []
        else:
            for count, msg in enumerate(messages):
                if msg.get("type") == "activity":
                    if msg.get("time") == datetime.now().isoformat().split('T')[0]:
                        words_count+=int(msg.get('msg'))
                    del(messages[count])
        activity_msg = { "type": "activity",
                "time": datetime.now().isoformat().split('T')[0],
                "msg": len(data.split())+words_count }
        messages.append(activity_msg)
        self.bot.db.setlist(key, messages)
        # all historical privmsg destroyed, and only one retained for seen
        # "42": {
        # "msg": "a b c d e",
        # "time": "2022-07-27T07:26:41.937448",
        # "type": "privmsg"
        # },
        key = "last_msg_for_{}".format(mask.nick).lower()
        priv_msg = { "type": "privmsg",
                "time": datetime.now().isoformat(),
                "msg": data }
        messages = self.bot.db.getlist(key)
        if not messages:
            messages = []
        else:
            for count, msg in enumerate(messages):
                if msg.get("type") == "privmsg":
                    del(messages[count])
        messages.append(priv_msg)
        self.bot.db.setlist(key, messages)
        return
    #######################################################################################
    #######################################################################################
    @command(permission='view')
    def seen(self, mask, target, args):
        """Display last time user was seen
            %%seen <nick>
        """
        nick = args.get("<nick>")
        if nick.lower()  == self.bot.get_nick().lower():
            msg = "please don't spy on me..."
            msg = self.bot.emo(msg)
            self.bot.privmsg(target, msg)
            return
        key = "last_msg_for_{}".format(nick).lower()
        message_list = self.bot.db.getlist(key)
        if message_list:
            message_list.sort(key=operator.itemgetter('time'), reverse=True)
            nick_seen_set = set()
            for msg in message_list:
                if msg.get("type") == "newnick":
                    notice_msg = msg.get("msg").lower()
                    nick_to_remember = [ item.strip() for item in notice_msg.split("changed nick to")].pop()
                    if nick_to_remember in nick_seen_set:
                        continue
                    else:
                        nick_seen_set.add(nick_to_remember)
                msg_time = dateutil.parser.parse(msg.get("time"))
                time_since = timeago.format(msg_time, datetime.now())
                msg = "\x02{}\x0F \x0303▶\x0F \x02\x0302{}\x0F \x0306፨ \x0F\x1F\x1D{}\x0F".format(nick, msg.get("msg"), time_since.capitalize())
                msg = self.bot.emo(msg)
                self.bot.privmsg(target, msg)
        else:
            msg = "{} Has Not Been Seen".format(nick)
            msg = self.bot.emo(msg)
            self.bot.privmsg(target, msg)
        return
    #######################################################################################
    #######################################################################################
###########################################################################################
###########################################################################################
